﻿using System;
using System.Collections;
using System.Linq;
using System.Windows.Controls;
using Microsoft.Phone.Controls.Primitives;

namespace LoopingSelectorSample.Controls
{
    /// <summary>
    /// 汎用的なコレクションを使用する <see cref="ILoopingSelectorDataSource"/> 実装クラスです。 
    /// </summary>
    public class LoopingSelectorItemsSource : ILoopingSelectorDataSource
    {
        /// <summary>
        /// 一覧の生成に使用するコレクションを取得または設定します。
        /// </summary>
        public IEnumerable ItemsSource { get; set; }

        /// <summary>
        /// 現在のオブジェクトが変更されたときに発生します。
        /// </summary>
        public event EventHandler<SelectionChangedEventArgs> SelectionChanged;

        private object _selectedItem;
        /// <summary>
        /// 現在のオブジェクトを取得または設定します。
        /// </summary>
        public object SelectedItem
        {
            get { return _selectedItem; }
            set
            {
                var oldValue = new[] { _selectedItem };
                var newValue = new[] { value };
                _selectedItem = value;
                OnSelectedChanged(new SelectionChangedEventArgs(oldValue, newValue));
            }
        }

        /// <summary>
        /// <para>コレクションの次のオブジェクトを返します。</para>
        /// <para>コレクションの末尾に位置している場合は、先頭のオブジェクトが返されます。</para>
        /// </summary>
        public object GetNext(object relativeTo)
        {
            object nextItem = null;
            if (ItemsSource != null)
            {
                var items = ItemsSource.Cast<object>().ToList();
                if (items.Any())
                {
                    var nextIndex = items.IndexOf(relativeTo) + 1;
                    nextItem = (nextIndex < items.Count) ? items[nextIndex] : items.First();
                }
            }
            return nextItem;
        }

        /// <summary>
        /// <para>コレクションの前のオブジェクトを返します。</para>
        /// <para>コレクションの先頭に位置している場合は、末尾のオブジェクトが返されます。</para>
        /// </summary>
        public object GetPrevious(object relativeTo)
        {
            object prevItem = null;
            if (ItemsSource != null)
            {
                var items = ItemsSource.Cast<object>().ToList();
                if (items.Any())
                {
                    var prevIndex = items.IndexOf(relativeTo) - 1;
                    prevItem = (0 <= prevIndex) ? items[prevIndex] : items.Last();
                }
            }
            return prevItem;
        }

        /// <summary>
        /// <see cref="SelectedChanged"/> イベントを発生させます。 
        /// </summary>
        protected virtual void OnSelectedChanged(SelectionChangedEventArgs e)
        {
            var handle = SelectionChanged;
            if (handle != null)
            {
                handle(this, e);
            }
        }
    }
}