﻿using System.Collections;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Interactivity;
using LoopingSelectorSample.Controls;
using Microsoft.Phone.Controls.Primitives;
using Microsoft.Practices.Prism.Interactivity;

namespace LoopingSelectorSample.Interactivity
{
    /// <summary>
    /// LoopingSelectorコントロールにデータソースをバインドするビヘイビアです。
    /// </summary>
    public class LoopingSelectorDataSourceBindingListener : Behavior<LoopingSelector>
    {
        /// <summary>
        /// <see cref="ItemsSourceBinding"/> バインディングリスナーを保持します。
        /// </summary>
        private BindingListener itemsSourceBindingListener;

        /// <summary>
        /// <see cref="SelectedItemBinding"/> バインディングリスナーを保持します。
        /// </summary>
        private BindingListener selectedItemBindingListener;

        /// <summary>
        /// <see cref="SelectedItemBinding"/> バインディングリスナーの依存関係プロパティを保持します。
        /// </summary>
        private DependencyProperty selectedItemBindingListenerProperty;

        /// <summary>
        /// <see cref="ItemsSourceBinding"/> 依存関係プロパティを識別します。
        /// </summary>
        public static readonly DependencyProperty ItemsSourceBindingProperty = DependencyProperty.RegisterAttached(
            "ItemsSourceBinding",
            typeof(Binding),
            typeof(LoopingSelectorDataSourceBindingListener),
            new PropertyMetadata((sender, e) =>
            {
                ((LoopingSelectorDataSourceBindingListener)sender).OnDataSourceBindingChanged(e);
            }));

        /// <summary>
        /// <see cref="SelectedItemBinding"/> 依存関係プロパティを識別します。
        /// </summary>
        public static readonly DependencyProperty SelectedItemBindingProperty = DependencyProperty.RegisterAttached(
            "SelectedItemBinding",
            typeof(Binding),
            typeof(LoopingSelectorDataSourceBindingListener),
            new PropertyMetadata((sender, e) =>
            {
                ((LoopingSelectorDataSourceBindingListener)sender).OnSelectedItemBindingChanged(e);
            }));

        /// <summary>
        /// 一覧の生成に使用するコレクションのバインド情報を取得または設定します。
        /// </summary>
        public Binding ItemsSourceBinding
        {
            get { return (Binding)GetValue(ItemsSourceBindingProperty); }
            set { SetValue(ItemsSourceBindingProperty, value); }
        }

        /// <summary>
        /// 現在のオブジェクトのバインド情報を取得または設定します。
        /// </summary>
        public Binding SelectedItemBinding
        {
            get { return (Binding)GetValue(SelectedItemBindingProperty); }
            set { SetValue(SelectedItemBindingProperty, value); }
        }

        /// <summary>
        /// <see cref="LoopingSelectorDataSourceBindingListener"/> クラスの新しいインスタンスを初期化します。
        /// </summary>
        public LoopingSelectorDataSourceBindingListener()
        {
            this.itemsSourceBindingListener = new BindingListener(ItemsSourceBindingValueChanged);
            this.selectedItemBindingListener = new BindingListener(SelectedItemBindingValueChanged);
        }

        /// <summary>
        /// LoopingSelectorコントロールにビヘイビアをアタッチします。
        /// </summary>
        protected override void OnAttached()
        {
            this.itemsSourceBindingListener.Element = base.AssociatedObject;
            this.selectedItemBindingListener.Element = base.AssociatedObject;

            base.AssociatedObject.DataSource = new LoopingSelectorItemsSource();
            base.AssociatedObject.DataSource.SelectionChanged += DataSource_SelectionChanged;

            base.OnAttached();
        }

        /// <summary>
        /// 依存関係プロパティとバインディングリスナーのバインド情報を同期します。
        /// </summary>
        private void OnDataSourceBindingChanged(DependencyPropertyChangedEventArgs e)
        {
            this.itemsSourceBindingListener.Binding = (Binding)e.NewValue;
        }

        /// <summary>
        /// 依存関係プロパティとバインディングリスナーのバインド情報を同期します。
        /// </summary>
        private void OnSelectedItemBindingChanged(DependencyPropertyChangedEventArgs e)
        {
            this.selectedItemBindingListener.Binding = (Binding)e.NewValue;
        }

        /// <summary>
        /// バインディングリスナーとデータソースの値を同期します。
        /// </summary>
        private void ItemsSourceBindingValueChanged(object sender, BindingChangedEventArgs e)
        {
            if (e.EventArgs.NewValue.Equals(e.EventArgs.OldValue) == false)
            {
                var dataSource = base.AssociatedObject.DataSource as LoopingSelectorItemsSource;
                dataSource.ItemsSource = e.EventArgs.NewValue as IEnumerable;
            }
        }

        /// <summary>
        /// バインディングリスナーとデータソースの値を同期します。
        /// </summary>
        private void SelectedItemBindingValueChanged(object sender, BindingChangedEventArgs e)
        {
            this.selectedItemBindingListenerProperty = e.EventArgs.Property;
            if (e.EventArgs.NewValue.Equals(e.EventArgs.OldValue) == false)
            {
                var dataSource = base.AssociatedObject.DataSource as LoopingSelectorItemsSource;
                dataSource.SelectedItem = e.EventArgs.NewValue;
            }
        }

        /// <summary>
        /// バインディングリスナーの変更を依存関係プロパティに反映します。
        /// </summary>
        private void DataSource_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems[0].Equals(e.RemovedItems[0]) == false)
            {
                base.AssociatedObject.SetValue(this.selectedItemBindingListenerProperty, e.AddedItems[0]);
            }
        }
    }
}